import { createSlice, PayloadAction } from '@reduxjs/toolkit';
import type { RootState } from './store';
import { AllLayoutAlgorithms, Layouts } from '@graphpolaris/shared/lib/graph-layout';
import { SchemaUtils } from '../../schema/schema-utils';
import { SchemaFromBackend, SchemaGraph, SchemaGraphology } from '../../schema';

/**************************************************************** */

export type SchemaSettings = {
  connectionType: 'connection' | 'bezier' | 'straight' | 'step';
  layoutName: AllLayoutAlgorithms;
  animatedEdges: boolean;
};

type schemaSliceI = {
  graph: SchemaGraph;
  settings: SchemaSettings;
};

// Define the initial state using that type
export const initialState: schemaSliceI = {
  graph: new SchemaGraphology().export(),
  // layoutName: 'Cytoscape_fcose',
  settings: {
    connectionType: 'connection',
    layoutName: Layouts.DAGRE,
    animatedEdges: false,
  },
};
export const schemaSlice = createSlice({
  name: 'schema',
  initialState,
  reducers: {
    setSchema: (state, action: PayloadAction<SchemaGraph>) => {
      if (action.payload === undefined) throw new Error('Schema is undefined');
      state.graph = action.payload;
    },
    clearSchema: (state) => {
      state.graph = new SchemaGraphology().export();
    },

    readInSchemaFromBackend: (state, action: PayloadAction<SchemaFromBackend>) => {
      state.graph = SchemaUtils.schemaBackend2Graphology(action.payload).export();
    },

    setSchemaSettings: (state, action: PayloadAction<SchemaSettings>) => {
      state.settings = action.payload;
    },
  },
});
export const { readInSchemaFromBackend, setSchema, setSchemaSettings, clearSchema } = schemaSlice.actions;

export const schemaSettingsState = (state: RootState) => state.schema.settings;

/**
 * Select the schema and convert it to a graphology object
 * */
export const toSchemaGraphology = (graph: SchemaGraph): SchemaGraphology => {
  // This is really weird but for some reason all the attributes appeared as read-only otherwise
  let ret = new SchemaGraphology();
  ret.import(SchemaGraphology.from(graph).export());
  return ret;
};

/**
 * Select the schema
 * */
export const schemaGraph = (state: RootState): SchemaGraph => {
  return state.schema.graph;
};

// /**
//  * selects the SchemaLayout enum
//  * @param {GraphLayout} state
//  * @returns {GraphLayout} enum of type GraphLayout
//  */
export const selectSchemaLayout = (state: RootState) => state.schema.settings.layoutName;

export default schemaSlice.reducer;
