import React from 'react';
import { DialogProps } from '@graphpolaris/shared/lib/components/Dialog';
import { useAppDispatch, useML } from '@graphpolaris/shared/lib/data-access';
import {
  setCentralityEnabled,
  setCommunityDetectionEnabled,
  setLinkPredictionEnabled,
  setShortestPathEnabled,
} from '@graphpolaris/shared/lib/data-access/store/mlSlice';
import { FormDiv, FormCard, FormBody, FormTitle, FormHBar } from '@graphpolaris/shared/lib/components/forms';

type QueryMLDialogProps = DialogProps;

export const QueryMLDialog = React.forwardRef<HTMLDivElement, QueryMLDialogProps>((props, ref) => {
  const dispatch = useAppDispatch();
  const ml = useML();

  return (
    <>
      {props.open && (
        <FormDiv ref={ref} className="" hAnchor="right">
          <FormCard>
            <FormBody
              onSubmit={(e) => {
                e.preventDefault();
              }}
            >
              <FormTitle title="Machine Learning Options" onClose={props.onClose} />
              <FormHBar />

              <div className="form-control px-5">
                <label className="label cursor-pointer gap-2 w-fit">
                  <input
                    type="checkbox"
                    checked={ml.linkPrediction.enabled}
                    className="checkbox checkbox-sm"
                    onChange={(e) => dispatch(setLinkPredictionEnabled(e.target.checked))}
                  />
                  <span className="label-text">Link Prediction</span>
                </label>
                {ml.linkPrediction.enabled && ml.linkPrediction.result && <span># of predictions: {ml.linkPrediction.result.length}</span>}
                {ml.linkPrediction.enabled && !ml.linkPrediction.result && <span>Loading...</span>}
                <label className="label cursor-pointer gap-2 w-fit">
                  <input
                    type="checkbox"
                    checked={ml.centrality.enabled}
                    className="checkbox checkbox-sm"
                    onChange={(e) => dispatch(setCentralityEnabled(e.target.checked))}
                  />
                  <span className="label-text">Centrality</span>
                </label>
                {ml.centrality.enabled && Object.values(ml.centrality.result).length > 0 && (
                  <span>
                    sum of centers:
                    {Object.values(ml.centrality.result)
                      .reduce((a, b) => b + a)
                      .toFixed(2)}
                  </span>
                )}
                {ml.centrality.enabled && Object.values(ml.centrality.result).length === 0 && <span>No Centers Found</span>}
                <label className="label cursor-pointer gap-2 w-fit">
                  <input
                    type="checkbox"
                    checked={ml.communityDetection.enabled}
                    className="checkbox checkbox-sm"
                    onChange={(e) => dispatch(setCommunityDetectionEnabled(e.target.checked))}
                  />
                  <span className="label-text">Community Detection</span>
                </label>
                {ml.communityDetection.enabled && ml.communityDetection.result && (
                  <span># of communities: {ml.communityDetection.result.length}</span>
                )}
                {ml.communityDetection.enabled && !ml.communityDetection.result && <span>Loading...</span>}
                <label className="label cursor-pointer gap-2 w-fit">
                  <input
                    type="checkbox"
                    checked={ml.shortestPath.enabled}
                    className="checkbox checkbox-sm"
                    onChange={(e) => dispatch(setShortestPathEnabled(e.target.checked))}
                  />
                  <span className="label-text">Shortest Path</span>
                </label>
                {ml.shortestPath.enabled && ml.shortestPath.result?.length > 0 && <span># of hops: {ml.shortestPath.result.length}</span>}
                {ml.shortestPath.enabled && !ml.shortestPath.srcNode && <span>Please select source node</span>}
                {ml.shortestPath.enabled && ml.shortestPath.srcNode && !ml.shortestPath.trtNode && <span>Please select target node</span>}
              </div>
            </FormBody>
          </FormCard>
        </FormDiv>
      )}
    </>
  );
});
