import React, { useState, useEffect } from 'react';
import { FormActions, FormBody, FormCard, FormControl, FormDiv, FormHBar, FormTitle } from '@graphpolaris/shared/lib/components/forms';
import { DialogProps } from '@graphpolaris/shared/lib/components/Dialog';
import { useAppDispatch, useVisualizationState } from '@graphpolaris/shared/lib/data-access';
import { updateGeneralSettings, updateVisualizationSettings } from '@graphpolaris/shared/lib/data-access/store/visualizationSlice';
import Input from '@graphpolaris/shared/lib/components/inputs';
import { globalConfigSchemaTypes, localConfigSchemaType } from '@graphpolaris/shared/lib/vis/Types';

export default function VisualizationDialog(props: DialogProps) {
  const dispatch = useAppDispatch();
  const vis = useVisualizationState();
  const [settingsGeneral, setSettingsGeneral] = useState<globalConfigSchemaTypes>(vis.settings.general);
  const [settingsSpecific, setSettingsSpecific] = useState<localConfigSchemaType>(vis.settings[vis.activeVisualization]);

  useEffect(() => {
    if (vis.settings.general !== settingsGeneral) {
      dispatch(updateGeneralSettings(settingsGeneral));
    }
  }, [settingsGeneral]);

  useEffect(() => {
    if (vis.settings[vis.activeVisualization] !== settingsSpecific) {
      dispatch(
        updateVisualizationSettings({
          id: vis.activeVisualization,
          settings: settingsSpecific,
        })
      );
    }
  }, [settingsSpecific]);

  useEffect(() => {
    setSettingsSpecific(vis.settings[vis.activeVisualization]);
  }, [vis.settings[vis.activeVisualization]]);

  return (
    <>
      {props.open && (
        <FormDiv>
          <FormCard>
            <FormBody
              onSubmit={(e) => {
                e.preventDefault();
                props.onClose();
              }}
            >
              <FormTitle title="Settings" onClose={props.onClose} />
              <FormHBar />

              {settingsGeneral && (
                <>
                  {Object.keys(settingsGeneral).map((val) => (
                    <FormControl key={val}>
                      <Input
                        {...settingsGeneral[val]}
                        value={vis.settings.general[val].value as any}
                        onChange={(value: any) => {
                          setSettingsGeneral((prevSettings) => ({
                            ...prevSettings,
                            [val]: { ...prevSettings[val], value: value },
                          }));
                        }}
                      />
                    </FormControl>
                  ))}
                </>
              )}

              {Object.keys(settingsSpecific).length > 0 && (
                <>
                  <FormHBar />
                  {Object.keys(settingsSpecific).map((val) => {
                    const currentSetting = settingsSpecific[val];
                    const shouldShowSetting = currentSetting.condition ? currentSetting.condition?.(settingsSpecific) : true;
                    return (
                      shouldShowSetting && (
                        <FormControl key={val}>
                          <Input
                            {...settingsSpecific[val]}
                            value={settingsSpecific[val]?.value as any}
                            onChange={(value: any) => {
                              setSettingsSpecific((prevSettings) => ({
                                ...prevSettings,
                                [val]: { ...prevSettings[val], value: value },
                              }));
                            }}
                          />
                        </FormControl>
                      )
                    );
                  })}
                </>
              )}

              <FormHBar />
              <FormActions onClose={props.onClose} />
            </FormBody>
          </FormCard>
        </FormDiv>
      )}
    </>
  );
}
