import React, { useEffect, useRef, useState } from 'react';
import { useImmer } from 'use-immer';
import { GraphQueryResult, useGraphQueryResult } from '../../../data-access/store';
import { ML } from '../../../data-access/store/mlSlice';
import { LinkType, NodeType } from './Types';
import { MatrixPixi } from './components/MatrixPixi';
import { VisualizationPropTypes, VISComponentType, localConfigSchemaType } from '../../Types';

interface Props {
  loading?: boolean;
  // currentColours: any;
}

// const displayName = 'Matrix Visualization';

export const MatrixVis = React.memo(({ data, ml, dispatch, localConfig }: VisualizationPropTypes<typeof displayName>) => {
  const ref = useRef<HTMLDivElement>(null);
  const [graph, setGraph] = useImmer<GraphQueryResult | undefined>(undefined);
  const [highlightNodes, setHighlightNodes] = useState<NodeType[]>([]);
  const [highlightedLinks, setHighlightedLinks] = useState<LinkType[]>([]);

  useEffect(() => {
    if (data) {
      setGraph(data);
    }
  }, [data, ml]);

  return (
    <>
      <div className="h-full w-full overflow-hidden" ref={ref}>
        <MatrixPixi graph={graph} highlightNodes={highlightNodes} highlightedLinks={highlightedLinks} localConfig={localConfig} />
      </div>
    </>
  );
});

const displayName = 'MatrixVis';
const localConfigSchema: localConfigSchemaType = {
  marks: {
    type: 'dropdown',
    options: ['rect', 'circle'],
    value: 'rect',
    label: 'Configure Marks',
  },
};

export const MatrixVisComponent: VISComponentType = {
  displayName: displayName,
  VIS: MatrixVis,
  localConfigSchema: localConfigSchema,
};
