import React, { useMemo, useRef } from 'react';
import { Table, AugmentedNodeAttributes } from './components/Table';
import { SchemaAttribute } from '../../../schema';
import { VisualizationPropTypes, VISComponentType, localConfigSchemaType } from '../../Types';

export type TableProps = {
  showBarplot: boolean;
  itemsPerPage: number;
};

const displayName = 'TableVis';

export const TableVis = ({ data, schema, localConfig }: VisualizationPropTypes<typeof displayName>) => {
  const ref = useRef<HTMLDivElement>(null);

  const attributesArray = useMemo<AugmentedNodeAttributes[]>(
    () =>
      data.nodes.map((node) => {
        const types: SchemaAttribute[] =
          schema.nodes.find((n) => n.key === node.label)?.attributes?.attributes ??
          schema.edges.find((r) => r.key === node.label)?.attributes?.attributes ??
          [];

        return {
          attribute: node.attributes,
          type: Object.fromEntries(types.map((t) => [t.name, t.type])),
        };
      }),
    [data.nodes]
  );

  return (
    <>
      <div className="h-full w-full" ref={ref}>
        {attributesArray.length > 0 && (
          <Table data={attributesArray} itemsPerPage={localConfig.itemsPerPage} showBarPlot={localConfig.showBarplot} />
        )}
      </div>
    </>
  );
};

const localConfigSchema: localConfigSchemaType = {
  showBarplot: {
    value: true,
    type: 'boolean',
    label: 'Show barplot',
  },
  itemsPerPage: {
    value: 10,
    type: 'dropdown',
    label: 'Items per page',
    options: [10, 20, 30],
  },
};

export const TableComponent: VISComponentType = {
  displayName: displayName,
  VIS: TableVis,
  localConfigSchema: localConfigSchema,
};
